//
//  ARView.h
//  ARApp2
//
//  Copyright 2010-2011 ARToolworks, Inc. All rights reserved.
//  Author: Philip Lamb
//

//
// ARView is a subclass of the standard EAGLView, which provides AR-related
// OpenGL drawing and utility functionality.
//
// Most importantly, it maps the video background into the OpenGL drawing
// context, and provides a skeleton OpenGL drawing loop.
//
// This version of ARMarker provides a set of drawing "hooks" (via NSNotification)
// to which other classes can connect to be informed of changes to the OpenGL viewport
// and projection and modelview matrices, and to draw their 3D content and 2D
// overlays at appropriate times in the frame drawing cycle.
//
// A single ARView is managed by a single ARViewController.
// 

#import "EAGLView.h"
#import "VEObject.h"

@class ARViewController;

/*
@protocol ARViewTouchDelegate<NSObject>
@optional
- (void) handleTouchAtLocation:(CGPoint)location tapCount:(NSUInteger)tapCount;
@end
*/

// Notifications.
extern NSString *const ARViewUpdatedCameraLensNotification;
extern NSString *const ARViewUpdatedCameraPoseNotification;
extern NSString *const ARViewUpdatedViewportNotification;
extern NSString *const ARViewDrawPreCameraNotification;
extern NSString *const ARViewDrawPostCameraNotification;
extern NSString *const ARViewDrawOverlayNotification;

enum viewPortIndices {
    viewPortIndexLeft = 0,
    viewPortIndexBottom,
    viewPortIndexWidth,
    viewPortIndexHeight
};

//@interface ARView : EAGLView <ARViewTouchDelegate> {
@interface ARView : EAGLView {

    ARViewController *arViewController;
    
    float cameraLens[16];
    float cameraPose[16];
    BOOL cameraPoseValid;
    int contentWidth;
    int contentHeight;
    BOOL contentRotate90;
    GLint viewPort[4];
    
    // Interaction.
    //id <ARViewTouchDelegate> touchDelegate;
    
    BOOL rayIsValid;
    float rayPoint1[3];
    float rayPoint2[3];
    
}

- (id) initWithFrame:(CGRect)frame pixelFormat:(NSString*)format depthFormat:(GLuint)depth stencilFormat:(GLuint)stencil preserveBackbuffer:(BOOL)retained;
@property float *cameraLens;
@property float *cameraPose;
@property(readonly) GLint *viewPort;
- (void) drawView;

// Points to the parent view controller.
@property (nonatomic, assign) IBOutlet ARViewController *arViewController;

// These properties allow variation on the way content is drawn in the GL window.
@property int contentWidth;
@property int contentHeight;
@property BOOL contentRotate90;

// Interaction.
@property(nonatomic, assign) id <ARViewTouchDelegate> touchDelegate;

@end
