//
//  VEObject.h
//  ARApp2
//
//  Copyright 2010-2011 ARToolworks, Inc. All rights reserved.
//  Author: Philip Lamb
//

//#import <Foundation/Foundation.h> // Using precompiled header.
#import <AR/ar.h>
#import "ARMarker.h"

@class VirtualEnvironment;
@protocol ARViewTouchDelegate;

extern NSString *const kVEObjectType;

@interface VEObject : NSObject {
    VirtualEnvironment *ve; // Reference to containing environment.
    BOOL drawable; // Whether this object is drawable.
    BOOL lit; // For drawable objects only, whether the object is currently drawn with lighting or without. Initial state is lit = TRUE.
    BOOL visible; // For drawable objects only, whether the object is currently hidden or visible. Initial state is visible = TRUE.
    ARPose pose; // Position and orientation of object, column-major order. (position(x,y,z) = {pose.T[12], pose.T[13], pose.T[14]}.)
    ARPose localPose; // Additional transform specifiying position and orientation of object in local coordinate system, column-major order.
    VEObject *parent;
    NSMutableArray *children;
    
    // Interaction.
    id <ARViewTouchDelegate> touchDelegate;
    float gDrawRotateAngle;
}

// Designated initialiser.
- (id)initWithVE:(VirtualEnvironment *)ve_in;

@property(readonly) NSString *type;
@property(readonly, getter=isDrawable) BOOL drawable;
@property(getter=isLit) BOOL lit;
@property(getter=isVisible) BOOL visible;
@property ARPose pose;
- (BOOL) isIntersectedByRayFromPoint:(ARVec3)p1 toPoint:(ARVec3)p2;

// VEObject hierachy.
@property(readonly) VEObject *parent;
@property(readonly) NSArray *children;
- (void) addChild:(VEObject *)child;
- (void) removeChild:(VEObject *)child;

// These messages are sent when an object is added to or removed from the VEObject hierachy.
// If overriding these, be sure to call through to super in your implementation.
- (void) willBeAddedTo:(VEObject *)parent;
- (void) wasAddedTo:(VEObject *)parent;
- (void) willBeRemovedFrom:(VEObject *)parent;
- (void) wasRemovedFrom:(VEObject *)parent;

// Selectors for handling notifications from ARMarker objects.
- (void) markerUpdatedPose:(NSNotification *)notification;
- (void) markerAppeared:(NSNotification *)notification;
- (void) markerDisappeared:(NSNotification *)notification;

//
// The following methods assume that position, orientation and scale factors
// are applied in the order position, orientation, scale. I.e. position and
// orientation are expressed in the coordinate system establish by 'pose' and
// scale is applied to in the rotated coordinate system 'localPose'.
// 
@property ARPose localPose;
- (void) localPosePositionX:(float *)x Y:(float *)y Z:(float *)z;
- (void) localPoseOrientationAngle:(float *)degrees axisX:(float *)x axisY:(float *)y axisZ:(float *)z;
- (void) localPoseScaleX:(float *)x Y:(float *)y Z:(float *)z;
- (void) setLocalPosePositionX:(float)x Y:(float)y Z:(float)z;
- (void) setLocalPoseOrientationDegrees:(float)degrees axisX:(float)x axisY:(float)y axisZ:(float)z;
- (void) setLocalPoseScaleX:(float)x Y:(float)y Z:(float)z;

// Interaction.
@property(nonatomic, assign) id <ARViewTouchDelegate> touchDelegate;

@end

@protocol ARViewTouchDelegate <NSObject>

@optional
//- (void)tapDetectingView:(ARView *)view gotSingleTapAtPoint:(CGPoint)tapPoint;
//- (void)tapDetectingView:(ARView *)view gotDoubleTapAtPoint:(CGPoint)tapPoint;
//- (void)tapDetectingView:(ARView *)view gotTwoFingerTapAtPoint:(CGPoint)tapPoint;
- (void)tapDetectingView:(UIView *)view touchesBegan:(NSSet*)touches withEvent:(UIEvent*)event;

@end
