//
//  VEObject.m
//  ARApp2
//
//  Copyright 2010-2011 ARToolworks, Inc. All rights reserved.
//  Author: Philip Lamb
//

#import "VEObject.h"
#import "VirtualEnvironment.h"
#import <Eden/EdenMath.h>

NSString *const kVEObjectType = @"VEObject";

@implementation VEObject
@synthesize drawable, lit, visible, pose, parent, children;
@synthesize localPose, touchDelegate;

- (id)initWithVE:(VirtualEnvironment *)ve_in
{
    if ((self = [super init])) {
        ve = ve_in;
        drawable = FALSE;
        lit = TRUE;
        visible = TRUE;
        children = [[NSMutableArray array] retain];
        pose = ARPoseUnity;
        localPose = ARPoseUnity;
    }
    return (self);
}

-(NSString *)type
{
    return (NSStringFromClass([self class]));
}

- (BOOL) isIntersectedByRayFromPoint:(ARVec3)p1 toPoint:(ARVec3)p2
{
    return (FALSE);
}

- (void) addChild:(VEObject *)child
{
    if (child->parent) {
        if (child->parent == self) return; // Already added.
        [child->parent removeChild:child]; // The child may have only one parent.
    }
    [child willBeAddedTo:self];
    [children addObject:child];
    child->parent = self;
    [child wasAddedTo:self];
}

- (void) removeChild:(VEObject *)child
{
    if (child->parent != self) return; 
    [child willBeRemovedFrom:self];
    [children removeObject:child];
    child->parent = nil;
    [child wasRemovedFrom:self];
}

- (void) willBeAddedTo:(VEObject *)parent
{
    
}

- (void) wasAddedTo:(VEObject *)parent
{
    
}

- (void) willBeRemovedFrom:(VEObject *)parent
{
    
}

- (void) wasRemovedFrom:(VEObject *)parent
{
    
}

- (void) markerUpdatedPose:(NSNotification *)notification
{
    //[self setPose:[((ARMarker *)[notification object]) pose]];
    ARPose newPose;
    
    EdenMathRotateMatrix(newPose.T, [((ARMarker *)[notification object]) pose].T, DTOR*gDrawRotateAngle, 0.0f, 1.0f, 0.0f);
    [self setPose:newPose];
}

- (void)updateWithTimeDelta:(NSTimeInterval)timeDelta
{
    //if (gDrawRotate) {
    gDrawRotateAngle += (float)timeDelta * 45.0f; // Rotate cube at 45 degrees per second.
    if (gDrawRotateAngle > 360.0f) gDrawRotateAngle -= 360.0f;
    //}
}

#pragma mark TapDetectingViewDelegate
- (void)tapDetectingView:(UIView*)view touchesBegan:(NSSet*)touches withEvent:(UIEvent*)event
{
    //[self updateWithTimeDelta:timeDelta];
    gDrawRotateAngle = !gDrawRotateAngle;
    NSLog(@"Test5");
}


- (void) markerAppeared:(NSNotification *)notification
{
    [self setVisible:TRUE];
}

- (void) markerDisappeared:(NSNotification *)notification
{
    [self setVisible:FALSE];
}

- (void) localPosePositionX:(float *)x Y:(float *)y Z:(float *)z
{
    if (x) *x = localPose.T[13];
    if (y) *y = localPose.T[14];
    if (z) *z = localPose.T[15];
}

- (void) localPoseOrientationAngle:(float *)degrees axisX:(float *)x axisY:(float *)y axisZ:(float *)z
{
    float tempX = localPose.T[6] - localPose.T[9];
    float tempY = localPose.T[8] - localPose.T[2];
    float tempZ = localPose.T[1] - localPose.T[4];
    float r = sqrtf(tempX*tempX + tempY*tempY + tempZ*tempZ);
    if (!r) {
        if (degrees) *degrees = 0.0f;
        if (x) *x = 1.0f;
        if (y) *y = 0.0f;
        if (z) *z = 0.0f;
    } else {
        float t = localPose.T[0] + localPose.T[5] + localPose.T[10];
        if (degrees) *degrees = RTOD*atan2(r, t - 1.0f);
        if (x) *x = tempX / r;
        if (y) *y = tempY / r;
        if (z) *z = tempZ / r;
    }
}

// Will give incorrect results if negative scale factors have been used, OR if skew factors
// have been manually set in localPose.
- (void) localPoseScaleX:(float *)x Y:(float *)y Z:(float *)z
{
    if (x) *x = sqrtf(localPose.T[0]*localPose.T[0] + localPose.T[1]*localPose.T[1] + localPose.T[2]*localPose.T[2]);
    if (y) *y = sqrtf(localPose.T[4]*localPose.T[4] + localPose.T[5]*localPose.T[5] + localPose.T[6]*localPose.T[9]);
    if (z) *z = sqrtf(localPose.T[8]*localPose.T[8] + localPose.T[9]*localPose.T[9] + localPose.T[10]*localPose.T[10]);
}

- (void) setLocalPosePositionX:(float)x Y:(float)y Z:(float)z
{
    localPose.T[13] = x;
    localPose.T[14] = y;
    localPose.T[15] = z;
}

- (void) setLocalPoseOrientationDegrees:(float)degrees axisX:(float)x axisY:(float)y axisZ:(float)z
{
    float px, py, pz, sx, sy, sz;
    [self localPosePositionX:&px Y:&py Z:&pz]; // Save position.
    [self localPoseScaleX:&sx Y:&sy Z:&sz]; // Save scale.
    EdenMathRotationMatrix(localPose.T, DTOR*degrees, x, y, z);
    [self setLocalPosePositionX:px Y:py Z:pz];
    [self setLocalPoseScaleX:sx Y:sy Z:sz];
}

- (void) setLocalPoseScaleX:(float)x Y:(float)y Z:(float)z
{
    float sx, sy, sz;
    [self localPoseScaleX:&sx Y:&sy Z:&sz]; // Get current scale.
    sx = x/sx; sy = y/sy; sz = z/sz; // Adjust for new scale.
    localPose.T[0] *= sx;
    localPose.T[1] *= sx;
    localPose.T[2] *= sx;
    localPose.T[4] *= sy;
    localPose.T[5] *= sy;
    localPose.T[6] *= sy;
    localPose.T[8] *= sz;
    localPose.T[9] *= sz;
    localPose.T[10] *= sz;
}

- (void) dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [children release];
    [super dealloc];
}

@end
